import { IOnInit } from 'angular'
import { AppService } from '../../services/app.service'
import { SessionStorage } from '../../helpers/session-storage'
import * as angular from 'angular'
import { Functions } from '../../helpers/functions'
import { StateService } from '../../services/state.service'
import { constants } from '../../constants'
import Swal from 'sweetalert2'
import { ConfigService } from '../../services/config.service'

class ShoppingCartListPage implements IOnInit {
    private selectedProducts: Array<any>
    private shoppingCart: Array<any> = []
    private items: Array<any>
    private selectedItem: any
    private activeEdit: boolean
    private activeIndex: number
    private paymentError: boolean
    private paymentLoading: boolean
    private openPayment: boolean
    private config: any

    constructor(
        public $scope,
        public $state,
        public $timeout,
        public $stateParams,
        public appService: AppService,
        public $document,
        public stateService: StateService,
        public configService: ConfigService
    ) {
    }

    async $onInit() {
        const params = this.stateService.getParams()
        this.shoppingCart = SessionStorage.getItem('shoppingCart') || []
        this.initialize()
        this.config = await this.configService.getConfigPayment(params.name)
    }



    editItem(item, index) {
        this.activeEdit = true
        this.activeIndex = index
        this.selectedItem = Functions.objectAssign(item)
    }

    saveEditItem(item) {
        let product = Functions.objectAssign(item)
        product.sizes = product.sizes.filter((size) => size.qtdValue > 0)
        if (product.sizes.reduce((result, size) => result + size.qtdValue, 0) === 0) {
            this.shoppingCart.splice(this.activeIndex, 1)
        } else {
            this.shoppingCart[this.activeIndex] = product
        }
        SessionStorage.setItem('shoppingCart', this.shoppingCart)
        delete this.activeIndex
        delete this.activeEdit
    }

    round(value) {
        const signal = value > 0 ? 1 : -1
        return (Math.floor(Math.round(Math.abs(value * 100))) / 100) * signal
    }

    totalValue() {
        return this.round((this.shoppingCart || []).reduce((value, product) => {
            return value + this.getTotal(product)
        }, 0))
    }

    countQtdValue(product) {
        return product.sizes.reduce((result, size) => result + size.qtdValue, 0)
    }

    getValue(product) {
        return product.valuePromo || product.value
    }

    getTotal(product) {
        return this.round(this.countQtdValue(product) * this.getValue(product))
    }

    removeItem(index) {
        this.$timeout(() => {
            this.shoppingCart.splice(index, 1)
            SessionStorage.setItem('shoppingCart', this.shoppingCart)
        })
    }

    getItemsDto(sizes, product) {
        return (sizes || []).map((size) => {
            return {
                color: product.colors[0].value,
                size: size.value,
                qnt: size.qtdValue
            }
        })
    }

    translateShoppingCart() {
        return this.shoppingCart.reduce((result, prod) => {
            const index = result.findIndex((product) => product.id === prod.productId)
            if (index === -1) {
                result.push({
                    id: prod.productId,
                    ref: prod.codeReference,
                    files: prod.files,
                    name: prod.name,
                    price: prod.value,
                    items: this.getItemsDto(prod.sizes, prod)
                })
            } else {
                result[index].items = result[index].items.concat(this.getItemsDto(prod.sizes, prod))
            }
            return result
        }, [])
    }

    openPaymentMethods() {
        this.openPayment = true
    }

    async prepareCheckout(order, paymentMethod: 'boleto' | 'credit_card') {
        if (paymentMethod) {
            const responseCheckout: any = await this.openCheckout(paymentMethod)
            if (responseCheckout && responseCheckout.token) {
                this.$timeout(async () => {
                    this.paymentLoading = true
                    const capture = {
                        token: responseCheckout.token,
                        amount: this.round(this.totalValue() * 100),
                        organization: this.$stateParams.name,
                        orderId: order.id
                    }
                    try {
                        await this.appService.capture(capture)
                        Swal.fire({
                            title: 'Pagamento realizado',
                            text: 'Seu pedido foi enviado a loja e o pagamento já foi aprovado.',
                            type: 'success',
                            confirmButtonText: 'Ok'
                        }).then(() => {
                            SessionStorage.setItem('shoppingCart', [])
                            this.$state.go('catalog', {
                                name: this.$stateParams.name,
                                phone: this.$stateParams.phone,
                                catalogId: this.$stateParams.catalogId,
                            })
                        })
                    } catch (e) {
                        this.paymentLoading = false
                        Swal.fire({
                            title: 'Oops',
                            text: 'Não foi possível finalizar o pagamento, tente novamente.',
                            type: 'error',
                            confirmButtonText: 'Ok'
                        })
                    }
                })
            }
        }
    }

    async finish(paymentMethod) {
        this.openPayment = false
        this.paymentLoading = true
        const params = this.stateService.getParams()
        const user = SessionStorage.getItem('user')

        const order = await this.appService.saveOrder(params.name, {
            value: this.shoppingCart.reduce((total, sc) => {
                return total += sc.sizes.reduce((result, size) => {
                    return result += size.qtdValue * sc.value
                }, 0)
            }, 0),
            createdAt: new Date().getTime(),
            opened: true,
            paid: false,
            customer: {
                document: user.document,
                name: user.name,
                phone: user.phoneNumber,
            },
            products: this.translateShoppingCart()
        })

        switch (paymentMethod) {
            case 'activeCreditCardPayment':
                await this.prepareCheckout(order, `credit_card`)
                break
            case 'activeBoletoPayment':
                await this.prepareCheckout(order, `boleto`)
                break
            default:
                Swal.fire({
                    title: 'Pedido enviado',
                    text: 'Seu pedido foi enviado a loja, aguarde nosso contato.',
                    type: 'success',
                    confirmButtonText: 'Ok'
                }).then(() => {
                    SessionStorage.setItem('shoppingCart', [])
                    this.$state.go('catalog', {
                        name: this.$stateParams.name,
                        phone: this.$stateParams.phone,
                        catalogId: this.$stateParams.catalogId,
                    })
                })
        }
    }

    openCheckout(paymentMethod) {
        return new Promise((resolve) => {
            this.paymentError = false
            const checkout = new window['PagarMeCheckout'].Checkout({
                encryption_key: process.env.NODE_ENV === 'development' ? constants.PAGAR_ME_KEY_DEV : constants.PAGAR_ME_KEY_PROD,
                success: (response) => {
                    resolve(response)
                },
                error: (err) => {
                    this.paymentLoading = false
                    this.$timeout(() => this.paymentError = true)
                },
            })
            checkout.open({
                uiColor: '#25D366',
                amount: this.round(this.totalValue() * 100),
                customerData: 'true',
                paymentMethods: paymentMethod,
                createToken: 'true',
                uiBackground: 'rgba(91, 82, 143, .9)',
                billing: {
                    name: 'Ciclano de Tal',
                    address: {
                        country: 'br',
                        state: 'SP',
                        city: 'São Paulo',
                        neighborhood: 'Fulanos bairro',
                        street: 'Rua dos fulanos',
                        street_number: '123',
                        zipcode: '05170060'
                    }
                },
                shipping: {
                    name: 'Ciclano de Tal',
                    fee: 12345,
                    delivery_date: '2017-12-25',
                    expedited: true,
                    address: {
                        country: 'br',
                        state: 'SP',
                        city: 'São Paulo',
                        neighborhood: 'Fulanos bairro',
                        street: 'Rua dos fulanos',
                        street_number: '123',
                        zipcode: '05170060'
                    }
                },
                items: this.shoppingCart.map((product) => {
                    return {
                        id: product.productId,
                        tangible: true,
                        title: product.name,
                        unit_price: this.round(this.getTotal(product) * 100),
                        quantity: this.countQtdValue(product),
                    }
                })
            })
        })
    }

    initialize() {
        this.$timeout(() => {
            let x: any
            angular.element('.list-wrapper .content-item')
                .on('touchstart', (e: any) => {
                    angular.element('.list-wrapper .content-item.open').css('left', '0px').removeClass('open')
                    angular.element(e.currentTarget).addClass('open')
                    x = e.originalEvent.targetTouches[0].pageX
                })
                .on('touchmove', (e: any) => {
                    angular.element(e.currentTarget).addClass('touched')
                    let change = e.originalEvent.targetTouches[0].pageX - x
                    change = Math.min(Math.max(-500, change), 0)
                    e.currentTarget.style.left = change + 'px'
                })
                .on('touchend', (e: any) => {
                    const element: any = angular.element(e.currentTarget)
                    element.removeClass('touched')
                    const left = parseInt(e.currentTarget.style.left)
                    let newLeft = '0px'
                    if (left < -150) {
                        newLeft = '-400px'
                        this.removeItem(element[0].getAttribute('index'))
                    } else if (left < -50) {
                        newLeft = '-100px'
                    }
                    element.animate({ left: newLeft }, 1)
                })
        })
    }

}

export { ShoppingCartListPage }
